package de.is24.realEstates.domain.energycertificate.validation;

import de.is24.realEstates.domain.RealEstateType;
import de.is24.realEstates.domain.energyCertificate.EnergyCertificateCreationDate;
import de.is24.realEstates.domain.living.BuildingEnergyRatingType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_CONSTRUCTION_DATE_UNKNOWN;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_ELECTRICITY_CONSUMPTION;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_ENERGY_CONSUMPTION_ELECTRICITY;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_ENERGY_CONSUMPTION_HEATING;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_ENERGY_EFFICENCY_CLASS;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_HEATING_CONSUMPTION;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_THERMAL_CHARACTERISTIC;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_THERMAL_CHARACTERISTIC_ELECTRICITY;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_THERMAL_CHARACTERISTIC_HEATING;
import static de.is24.realEstates.domain.energycertificate.validation.EnergyCertificateField.EV_THERMAL_CHARACTERISTIC_WARM_WATER_INC;
import static de.is24.realEstates.domain.RealEstateType.APARTMENT_BUY;
import static de.is24.realEstates.domain.RealEstateType.APARTMENT_RENT;
import static de.is24.realEstates.domain.RealEstateType.ASSISTED_LIVING;
import static de.is24.realEstates.domain.RealEstateType.COMPULSORY_AUCTION;
import static de.is24.realEstates.domain.RealEstateType.FLAT_SHARE_ROOM;
import static de.is24.realEstates.domain.RealEstateType.GASTRONOMY;
import static de.is24.realEstates.domain.RealEstateType.HOUSE_BUY;
import static de.is24.realEstates.domain.RealEstateType.HOUSE_RENT;
import static de.is24.realEstates.domain.RealEstateType.INDUSTRY;
import static de.is24.realEstates.domain.RealEstateType.INVESTMENT;
import static de.is24.realEstates.domain.RealEstateType.OFFICE;
import static de.is24.realEstates.domain.RealEstateType.SHORT_TERM_ACCOMMODATION;
import static de.is24.realEstates.domain.RealEstateType.SPECIAL_PURPOSE;
import static de.is24.realEstates.domain.RealEstateType.STORE;


/**
 * Created by kfolwarczny on 3/25/14.
 */
public class EnergyCertificateFieldsValidation {
  private static final Logger LOGGER = LoggerFactory.getLogger(EnergyCertificateFieldsValidation.class);

  private static final Map<RealEstateType, EnergyCertificateFieldCombination> FIELDS_PER_TYPE =
    new HashMap<RealEstateType, EnergyCertificateFieldCombination>() {
      {
        // Haus Kauf
        put(HOUSE_BUY,
          getFieldsForLivingRealEstates());

        // Wohnung Kauf
        put(APARTMENT_BUY,
          getFieldsForLivingRealEstates());

        //  Anlage
        put(INVESTMENT, getFieldsForInvestmentProperty());

        // Haus Miete
        put(HOUSE_RENT, getFieldsForLivingRealEstates());

        // Wohnung Miete
        put(APARTMENT_RENT, getFieldsForLivingRealEstates());

        // WG Zimmer
        put(FLAT_SHARE_ROOM, getFieldsForLivingRealEstates());

        // Wohnen auf Zeit
        put(SHORT_TERM_ACCOMMODATION, getFieldsForLivingRealEstates());

        // SeniorenWohnen
        put(ASSISTED_LIVING,
          getFieldsForLivingRealEstates());

        // Zwangsversteigerung
        put(COMPULSORY_AUCTION, getFieldsForCompulsoryAuction());

        // Büro/Praxen
        put(OFFICE, getFieldsForCommercialRealEstates());

        // Einzelhandel
        put(STORE, getFieldsForCommercialRealEstates());

        // Hallen/Produktion
        put(INDUSTRY, getFieldsForCommercialRealEstates());

        // Gastronomie/Hotel
        put(GASTRONOMY, getFieldsForCommercialRealEstates());

        // Spezialgewerbe
        put(SPECIAL_PURPOSE, getFieldsForCommercialRealEstates());

      }

      private EnergyCertificateFieldCombination getFieldsForInvestmentProperty() {
        Set<EnergyCertificateField> demandBeforeFirstMay = EnumSet.of(EV_THERMAL_CHARACTERISTIC);

        Set<EnergyCertificateField> demandAfterFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_THERMAL_CHARACTERISTIC_ELECTRICITY,
          EV_THERMAL_CHARACTERISTIC_HEATING,
          EV_ENERGY_EFFICENCY_CLASS);

        Set<EnergyCertificateField> consumptionBeforeFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_THERMAL_CHARACTERISTIC_WARM_WATER_INC,
          EV_ELECTRICITY_CONSUMPTION,
          EV_HEATING_CONSUMPTION);

        Set<EnergyCertificateField> consumptionAfterFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_ENERGY_EFFICENCY_CLASS,
          EV_ENERGY_CONSUMPTION_ELECTRICITY,
          EV_ENERGY_CONSUMPTION_HEATING);

        return new EnergyCertificateFieldCombination(demandAfterFirstMay, consumptionAfterFirstMay,
          demandBeforeFirstMay, consumptionBeforeFirstMay);
      }

      private EnergyCertificateFieldCombination getFieldsForCompulsoryAuction() {
        Set<EnergyCertificateField> demandBeforeFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> demandAfterFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_THERMAL_CHARACTERISTIC_ELECTRICITY,
          EV_THERMAL_CHARACTERISTIC_HEATING,
          EV_ENERGY_EFFICENCY_CLASS,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> consumptionBeforeFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_THERMAL_CHARACTERISTIC_WARM_WATER_INC,
          EV_ELECTRICITY_CONSUMPTION,
          EV_HEATING_CONSUMPTION,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> consumptionAfterFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_ENERGY_EFFICENCY_CLASS,
          EV_ENERGY_CONSUMPTION_ELECTRICITY,
          EV_ENERGY_CONSUMPTION_HEATING,
          EV_CONSTRUCTION_DATE_UNKNOWN);

        return new EnergyCertificateFieldCombination(demandAfterFirstMay, consumptionAfterFirstMay,
          demandBeforeFirstMay, consumptionBeforeFirstMay);
      }

      private EnergyCertificateFieldCombination getFieldsForCommercialRealEstates() {
        Set<EnergyCertificateField> demandBeforeFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> demandAfterFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_THERMAL_CHARACTERISTIC_ELECTRICITY,
          EV_THERMAL_CHARACTERISTIC_HEATING,
          EV_ENERGY_EFFICENCY_CLASS,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> consumptionBeforeFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_THERMAL_CHARACTERISTIC_WARM_WATER_INC,
          EV_ELECTRICITY_CONSUMPTION,
          EV_HEATING_CONSUMPTION,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> consumptionAfterFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_ENERGY_EFFICENCY_CLASS,
          EV_ENERGY_CONSUMPTION_ELECTRICITY,
          EV_ENERGY_CONSUMPTION_HEATING,
          EV_CONSTRUCTION_DATE_UNKNOWN);

        return new EnergyCertificateFieldCombination(demandAfterFirstMay, consumptionAfterFirstMay,
          demandBeforeFirstMay, consumptionBeforeFirstMay);
      }

      private EnergyCertificateFieldCombination getFieldsForLivingRealEstates() {
        Set<EnergyCertificateField> demandBeforeFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> demandAfterFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_ENERGY_EFFICENCY_CLASS,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> consumptionBeforeFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_THERMAL_CHARACTERISTIC_WARM_WATER_INC,
          EV_CONSTRUCTION_DATE_UNKNOWN);
        Set<EnergyCertificateField> consumptionAfterFirstMay = EnumSet.of(
          EV_THERMAL_CHARACTERISTIC,
          EV_ENERGY_EFFICENCY_CLASS,
          EV_CONSTRUCTION_DATE_UNKNOWN);

        return new EnergyCertificateFieldCombination(demandAfterFirstMay, consumptionAfterFirstMay,
          demandBeforeFirstMay, consumptionBeforeFirstMay);
      }
    };

  public static Set<EnergyCertificateField> getSetOfFieldsForITypeDateAndCertType(
    RealEstateType realEstateType, BuildingEnergyRatingType buildingEnergyRatingType,
    EnergyCertificateCreationDate energyCertificateCreationDate) {
    final EnergyCertificateFieldCombination energyCertificateFieldCombination = FIELDS_PER_TYPE.get(realEstateType);

    if (energyCertificateFieldCombination == null) {
      LOGGER.error("For key: " + realEstateType + ", map does not have any Objects!");
      return Collections.emptySet();
    }

    if ((buildingEnergyRatingType == null) || (energyCertificateCreationDate == null)) {
      LOGGER.error("BuildingEnergyRatingType or EnergyCertificateCreationDate was passed as null");
      return Collections.emptySet();
    }


    return energyCertificateFieldCombination.getValidSetForEVTypeAndDate(buildingEnergyRatingType,
      energyCertificateCreationDate);
  }

  public static Set<EnergyCertificateField> getBasicFieldSetForImmoType(RealEstateType realEstateType) {
    final EnergyCertificateFieldCombination energyCertificateFieldCombination = FIELDS_PER_TYPE.get(realEstateType);

    if (energyCertificateFieldCombination == null) {
      LOGGER.error("For key: " + realEstateType + ", map does not have any Objects!");
      return Collections.emptySet();
    }

    return energyCertificateFieldCombination.getBasicFieldSet();
  }
}
